/**
 * @author      Takashi Matsushita
 * Created:     9 Nov 2015
 */

/** @file */
/** @todo nope */

#ifndef tmEventSetup_esTypes_hh
#define tmEventSetup_esTypes_hh

/*====================================================================*
 * declarations
 *====================================================================*/
/*-----------------------------------------------------------------*
 * headers
 *-----------------------------------------------------------------*/
/* nope */


namespace tmeventsetup
{
/*-----------------------------------------------------------------*
 * constants
 *-----------------------------------------------------------------*/
const int Undef = -9999; /**< undefined value for enums */

const char ET_THR[] = "ET"; /**< text for Et threshold */

const char COUNT[] = "COUNT";     /**< text for Minimum Bias HF type  */

const char PRECISION[] = "PRECISION";     /**< text for precision type */
const char PRECISION_DELTA[] = "Delta";   /**< text for delta precision type */
const char PRECISION_MASS[] = "Mass";     /**< text for mass precision type */
const char PRECISION_MASSPT[] = "MassPt"; /**< text for mass precision type */
const char PRECISION_MATH[] = "Math";     /**< text for math precision type */


/** type of cuts
 *  \warning Never change the order
 */
enum esCutType
{
/*  0 */  Threshold,        /**< Et/Pt threshold */
/*  1 */  Eta,              /**< eta cut */
/*  2 */  Phi,              /**< phi cut */
/*  3 */  Charge,           /**< charge cut */
/*  4 */  Quality,          /**< quality cut */
/*  5 */  Isolation,        /**< isolation cut */
/*  6 */  DeltaEta,         /**< delta eta cut for correlation function */
/*  7 */  DeltaPhi,         /**< delta phi cut for correlation function */
/*  8 */  DeltaR,           /**< delta R cut for correlation function */
/*  9 */  Mass,             /**< invariant mass cut for mass function */
/* 10 */  ChargeCorrelation,/**< charge correlation cut for comb function */
/* 11 */  Count,            /**< number of counts */
/* 12 */  Index,            /**< specify indicies of objects to use */
/* 13 */  OverlapRemovalDeltaEta,  /**< delta eta cut for overlap removal */
/* 14 */  OverlapRemovalDeltaPhi,  /**< delta phi cut for overlap removal */
/* 15 */  OverlapRemovalDeltaR,    /**< delta R cut for overlap removal */
/* 16 */  nCutType
};

/** type of objects
 *  \warning Never change the numbering
 */
enum esObjectType
{
  Muon      = 0,  /**< muon */
  Egamma    = 1,  /**< electron/gamma */
  Tau       = 2,  /**< tau */
  Jet       = 3,  /**< jet */
  ETT       = 4,  /**< total Et */
  HTT       = 5,  /**< total hadronic Et */
  ETM       = 6,  /**< missing Et */
  HTM       = 7,  /**< missing hadronic Et */
  EXT       = 8,  /**< extrnals */
  Precision = 9,  /**< precision type */
  MBT0HFP   = 13, /**< Minimum Bias HF+ threshold 0 */
  MBT1HFP   = 14, /**< Minimum Bias HF+ threshold 1 */
  MBT0HFM   = 15, /**< Minimum Bias HF- threshold 0 */
  MBT1HFM   = 16, /**< Minimum Bias HF- threshold 1 */
  ETTEM     = 17, /**< total Et with ECAL only */
  ETMHF     = 18, /**< missing Et with HF */
  TOWERCOUNT = 19, /**< missing Et with HF */
  nObjectType = 17
};

/** type of Function
 *  \warning Never change the numbering
 */
enum esFunctionType
{
  CombFunction = 10, /**< combination */
  DistFunction = 11, /**< distance */
  MassFunction = 12, /**< invariant mass */
  Function = 20,     /**< generic function */
  FunctionWithOverlapRemoval = 21,     /**< generic function */
  CombFunctionWithOverlapRemoval = 22, /**< combination with overlap removal */
  DistFunctionWithOverlapRemoval = 23, /**< distance with overlap removal */
  MassFunctionWithOverlapRemoval = 24  /**< invariant mass with overlap removal */
};

/** list of comparison operator
 *  \warning Never change the order
 */
enum esComparisonOperator
{
/* 0 */  GE, /**< greater than or equal to */
/* 1 */  NE, /**< not equal to */
/* 2 */  EQ  /**< equal to */
};

/* type of combinations
 *  \warning Never change the order
 */
enum esCombinationType
{
/* 0 */  MuonMuonCombination,    /**< Muon-Muon combination */
/* 1 */  MuonEsumCombination,    /**< Muon-Esum combination */
/* 2 */  CaloMuonCombination,    /**< Calo-Muon combination */
/* 3 */  CaloCaloCombination,    /**< Calo-Calo combination */
/* 4 */  CaloEsumCombination,    /**< Calo-Esum combination */
/* 5 */  nCombinationType
};

/* type of conditinos
 *  \warning Never change the order
 */
enum esConditionType
{
/*  0 */  SingleMuon,   /**< single muon condition */
/*  1 */  DoubleMuon,   /**< double muon condition */
/*  2 */  TripleMuon,   /**< triple muon condition */
/*  3 */  QuadMuon,     /**< quad muon condition */
/*  4 */  SingleEgamma, /**< single e/gamma condition */
/*  5 */  DoubleEgamma, /**< double e/gamma condition */
/*  6 */  TripleEgamma, /**< triple e/gamma condition */
/*  7 */  QuadEgamma,   /**< quad e/gamma condition */
/*  8 */  SingleTau,    /**< single tau condition */
/*  9 */  DoubleTau,    /**< double tau condition */
/* 10 */  TripleTau,    /**< triple tau condition */
/* 11 */  QuadTau,      /**< quad tau condition */
/* 12 */  SingleJet,    /**< single jet condition */
/* 13 */  DoubleJet,    /**< double jet condition */
/* 14 */  TripleJet,    /**< triple jet condition */
/* 15 */  QuadJet,      /**< quad jet condition */
/* 16 */  TotalEt,      /**< total Et condition */
/* 17 */  TotalHt,      /**< total Ht condition */
/* 18 */  MissingEt,    /**< missing Et condition */
/* 19 */  MissingHt,    /**< missing Ht condition */
/* 20 */  Externals,    /**< external condition */
/* 21 */  MuonMuonCorrelation,    /**< Muon-Muon correlation condition */
/* 22 */  MuonEsumCorrelation,    /**< Muon-Esum correlation condition */
/* 23 */  CaloMuonCorrelation,    /**< Calo-Muon correlation condition */
/* 24 */  CaloCaloCorrelation,    /**< Calo-Calo correlation condition */
/* 25 */  CaloEsumCorrelation,    /**< Calo-Esum correlation condition */
/* 26 */  InvariantMass,          /**< Invariant mass condition */
/* 27 */  MinBiasHFP0,  /**< Minimum Bias HF+ threshold 0 */
/* 28 */  MinBiasHFP1,  /**< Minimum Bias HF+ threshold 1 */
/* 29 */  MinBiasHFM0,  /**< Minimum Bias HF+ threshold 0 */
/* 30 */  MinBiasHFM1,  /**< Minimum Bias HF+ threshold 1 */
/* 31 */  TotalEtEM,    /**< total Et with ECAL only condition */
/* 32 */  MissingEtHF,  /**< missing Et with HF condition */
/* 33 */  TowerCount,   /**< Calo tower count */
  // overlap removals
/* 34 */  SingleMuonWithOverlapRemoval,   /**< single muon condition with overlap removal */
/* 35 */  DoubleMuonWithOverlapRemoval,   /**< double muon condition with overlap removal */
/* 36 */  TripleMuonWithOverlapRemoval,   /**< triple muon condition with overlap removal */
/* 37 */  QuadMuonWithOverlapRemoval,     /**< quad muon condition with overlap removal */
/* 38 */  SingleEgammaWithOverlapRemoval, /**< single e/gamma condition with overlap removal */
/* 39 */  DoubleEgammaWithOverlapRemoval, /**< double e/gamma condition with overlap removal */
/* 40 */  TripleEgammaWithOverlapRemoval, /**< triple e/gamma condition with overlap removal */
/* 41 */  QuadEgammaWithOverlapRemoval,   /**< quad e/gamma condition with overlap removal */
/* 42 */  SingleTauWithOverlapRemoval,    /**< single tau condition with overlap removal */
/* 43 */  DoubleTauWithOverlapRemoval,    /**< double tau condition with overlap removal */
/* 44 */  TripleTauWithOverlapRemoval,    /**< triple tau condition with overlap removal */
/* 45 */  QuadTauWithOverlapRemoval,      /**< quad tau condition with overlap removal */
/* 46 */  SingleJetWithOverlapRemoval,    /**< single jet condition with overlap removal */
/* 47 */  DoubleJetWithOverlapRemoval,    /**< double jet condition with overlap removal */
/* 48 */  TripleJetWithOverlapRemoval,    /**< triple jet condition with overlap removal */
/* 49 */  QuadJetWithOverlapRemoval,      /**< quad jet condition with overlap removal */
/* 50 */  MuonMuonCorrelationWithOverlapRemoval,    /**< Muon-Muon correlation condition with overlap removal */
/* 51 */  MuonEsumCorrelationWithOverlapRemoval,    /**< Muon-Esum correlation condition with overlap removal */
/* 52 */  CaloMuonCorrelationWithOverlapRemoval,    /**< Calo-Muon correlation condition with overlap removal */
/* 53 */  CaloCaloCorrelationWithOverlapRemoval,    /**< Calo-Calo correlation condition with overlap removal */
/* 54 */  CaloEsumCorrelationWithOverlapRemoval,    /**< Calo-Esum correlation condition with overlap removal */
/* 55 */  InvariantMassWithOverlapRemoval,          /**< Invariant mass condition with overlap removal */
/* 56 */  nConditionType
};

/** type of scales
 *  \warning Never change the order
 */
enum esScaleType
{
/*  0 */  EtScale,          /**< Et scale */
/*  1 */  EtaScale,         /**< eta scale */
/*  2 */  PhiScale,         /**< phi scale */
/*  3 */  DeltaPrecision,   /**< precision for delta */
/*  4 */  MassPrecision,    /**< precision for mass */
/*  5 */  MassPtPrecision,  /**< precision for mass pt */
/*  6 */  MathPrecision,    /**< precision for math */
/*  7 */  CountScale        /**< Count scale */
};

} // namespace tmeventsetup
#endif // tmEventSetup_esTypes_hh
/* eof */
